import matplotlib.pyplot as plt
import numpy as np
from scipy.integrate import odeint
plt.close()

v0=10
g=9.81
theta=np.radians(50)

#Calul exact de la trajectoire en chute libre

def position(t):
    assert t>=0
    return v0*np.cos(theta)*t,-0.5*g*t**2+v0*np.sin(theta)*t  
     
        
t=0
pas=1e-1
OM=[0,0]
x,y=[OM[0]],[OM[1]]
while OM[1]>0 or len(x)==1:
    OM=position(t+pas)
    if OM[1]>0:
        x.append(OM[0])
        y.append(OM[1])
    t+=pas
plt.plot(x,y)
plt.show()
  
#Résolution de l'équation différentielle par la méthode d'Euler

def mvt(Y,t):
    return np.array([Y[2],Y[3],0,-g])

def euler(f,a,y0,pas):
    y=y0
    t=a
    liste_y=[y0]
    liste_t=[a]
    while y[1]>=0:
        y=y+pas*f(y,t)
        if y[1]<0:
            return liste_t,liste_y
        liste_y.append(y)
        t=t+pas
        liste_t.append(t)
    

CI=[0,0,v0*np.cos(theta),v0*np.sin(theta)]
t,Ye=euler(mvt,0,CI,pas)

xe,ye=[],[]
for i in range(len(t)):
    xe.append(Ye[i][0])
    ye.append(Ye[i][1])  
    
# Résolution à l'aide de la fonction odeint
 
Yode=odeint(mvt,CI,np.array(t))    
    
xode,yode=[],[]
for i in range(len(t)):
    xode.append(Yode[i][0])
    yode.append(Yode[i][1])
 

# Résolution par la méthode de Runge-Kutta 2 (Heun)

def RK2(f,a,y0,pas):
    y=y0
    t=a
    liste_y=[y0]
    liste_t=[a]
    while y[1]>=0:
        k1=f(y,t)
        k2=f(y+pas*k1,t+pas)
        y=y+0.5*pas*(k1+k2)
        if y[1]<0:
            return liste_t,liste_y
        liste_y.append(y)
        liste_t.append(t+pas)
    

CI=[0,0,v0*np.cos(theta),v0*np.sin(theta)]
t,Y=RK2(mvt,0,CI,pas)

xRK2,yRK2=[],[]
for i in range(len(t)):
    xRK2.append(Y[i][0])
    yRK2.append(Y[i][1])


# Résolution par la méthode de Runge-Kutta 4

def RK4(f,a,y0,pas):
    y=y0
    t=a
    liste_y=[y0]
    liste_t=[a]
    while y[1]>=0:
        k1=f(y,t)
        k2=f(y+0.5*pas*k1,t+0.5*pas)
        k3=f(y+0.5*pas*k2,t+0.5*pas)
        k4=f(y+pas*k3,t+pas)
        y=y+pas*(k1/6+k2/3+k3/3+k4/6)
        if y[1]<0:
            return liste_t,liste_y
        liste_y.append(y)
        liste_t.append(t+pas)

CI=[0,0,v0*np.cos(theta),v0*np.sin(theta)]
t,Y=RK4(mvt,0,CI,pas)

xRK4,yRK4=[],[]
for i in range(len(t)):
    xRK4.append(Y[i][0])
    yRK4.append(Y[i][1])
 
plt.plot(x,y,'--b',label='Sol. exacte')
plt.plot(xe,ye,'.r',label='Euler')
plt.plot(xode,yode,'.g',label='Odeint')
plt.plot(xRK2,yRK2,'.k',label='RK2')
plt.plot(xRK4,yRK4,'.m',label='RK4')
plt.legend()
plt.xlabel('x')
plt.ylabel('y')
plt.show()

# Erreur sur le calcul de la flèche
erreurFlecheEuler=[]
erreurFlecheOdeint=[]
erreurFlecheRK2=[]
erreurFlecheRK4=[]

CI=[0,0,v0*np.cos(theta),v0*np.sin(theta)]

h=np.arange(1e-3,1e-1,1e-3)

for pas in h:
    #calcul de y
    t=0
    y=[0]
    while y[-1]>=0:
        y.append(position(t+pas)[1])
        t+=pas
    #calcul de ye
    t,Y=euler(mvt,0,CI,pas)
    ye=[]
    for i in range(len(t)):
        ye.append(Y[i][1])
    #calcul de yode
    Yode=odeint(mvt,CI,np.array(t))    
    yode=[]
    for i in range(len(t)):
        yode.append(Yode[i][1])
    #calcul de yRK2
    t,Y=RK2(mvt,0,CI,pas)
    yRK2=[]
    for i in range(len(t)):
        yRK2.append(Y[i][1])
    #calcul de yRK2
    t,Y=RK4(mvt,0,CI,pas)
    yRK4=[]
    for i in range(len(t)):
        yRK4.append(Y[i][1])  
        
    erreurFlecheEuler.append(abs(max(ye)-max(y)))
    erreurFlecheOdeint.append(abs(max(yode)-max(y)))
    erreurFlecheRK2.append(abs(max(yRK2)-max(y)))
    erreurFlecheRK4.append(abs(max(yRK4)-max(y)))
    

plt.plot(h,erreurFlecheEuler,'.r',label='Euler')
plt.plot(h,erreurFlecheOdeint,'.g',label='Odeint')
plt.plot(h,erreurFlecheRK2,'.k',label='RK2')
plt.plot(h,erreurFlecheRK4,'.m',label='RK4')
plt.legend()
plt.xlabel('h')
plt.ylabel('Erreur')
plt.title('Erreur comise pour le calcul de la flèche en fonction du pas de calcul')
plt.show()


k=1
V=2
pas=1e-2

def mvtfrot(Y,t):
    return np.array([Y[2],Y[3],-k*(Y[2]**3/np.sqrt(Y[2]**2+Y[3]**2)+V**2),-g-k*Y[3]**3/np.sqrt(Y[2]**2+Y[3]**2)])

t,Yfrot=RK2(mvtfrot,0,CI,pas)

xfrotRK2,yfrotRK2=[],[]
for i in range(len(t)):
    xfrotRK2.append(Yfrot[i][0])
    yfrotRK2.append(Yfrot[i][1])
    

plt.plot(xfrotRK2,yfrotRK2,'.k',label='RK2')
plt.legend()
plt.xlabel('x')
plt.ylabel('y')
plt.show()